//: Playground - noun: a place where people can play

extension Graphable {
    public func route(to destination: Vertex<Element>, in tree: [Vertex<Element> : Visit<Element>]) -> [Edge<Element>] {
        
        var vertex = destination
        var path : [Edge<Element>] = []
        
        while let visit = tree[vertex],
            case .edge(let edge) = visit {
                
                path = [edge] + path
                vertex = edge.source
        }
        return path
    }
    
    public func distance(to destination: Vertex<Element>, in tree: [Vertex<Element> : Visit<Element>]) -> Double {
        let path = route(to: destination, in: tree)
        let distances = path.flatMap{ $0.weight }
        return distances.reduce(0.0, { $0 + $1 })
    }
    
	public func dijkstra(from source: Vertex<Element>, to destination: Vertex<Element>) -> [Edge<Element>]? {
        var visits : [Vertex<Element> : Visit<Element>] = [source: .source]
        var priorityQueue = PriorityQueue<Vertex<Element>>(sort: { self.distance(to: $0, in: visits) < self.distance(to: $1, in: visits) })
        priorityQueue.enqueue(source)
		
		while let visitedVertex = priorityQueue.dequeue() {
			if visitedVertex == destination {
                return route(to: destination, in: visits)
            }
            let neighbourEdges = edges(from: visitedVertex) ?? []
            for edge in neighbourEdges {
                if let weight = edge.weight {
                    if visits[edge.destination] != nil {
                        if distance(to: visitedVertex, in: visits) + weight < distance(to: edge.destination, in: visits)
                        {
                            visits[edge.destination] = .edge(edge)
                            priorityQueue.enqueue(edge.destination)
                        }
                    }
                    else {
                        visits[edge.destination] = .edge(edge)
                        priorityQueue.enqueue(edge.destination)
                    }
                }
            }
		}
		return nil
	}
}

let dungeon = AdjacencyList<String>()

let entranceRoom = dungeon.createVertex(data: "Entrance")
let spiderRoom = dungeon.createVertex(data: "Spider")
let goblinRoom = dungeon.createVertex(data: "Goblin")
let ratRoom = dungeon.createVertex(data: "Rat")
let treasureRoom = dungeon.createVertex(data: "Treasure")

dungeon.add(.undirected, from: entranceRoom, to: spiderRoom, weight: 11)
dungeon.add(.undirected, from: spiderRoom, to: goblinRoom, weight: 11)
dungeon.add(.undirected, from: goblinRoom, to: treasureRoom, weight: 11)
dungeon.add(.undirected, from: entranceRoom, to: ratRoom, weight: 31)
dungeon.add(.undirected, from: ratRoom, to: treasureRoom, weight: 12)

dungeon.description


if let edges = dungeon.dijkstra(from: entranceRoom, to: treasureRoom) {
	for edge in edges {
		print("\(edge.source) -> \(edge.destination)")
	}
}
